@@ -157,7 +157,7 @@ setMethod(".gtree",
             if (!is.null(container)) {
               if(is.logical(container) && container == TRUE)
                 container = gwindow(visible=TRUE)
-              add(container, obj)
+              add(container, obj,...)
             }
             
             return(obj)
